#!/bin/sh

#
# two groups of TSIs are known:
#   TSI without batch sub-system: composed from tsi/SHARED and
#                                 specific files from tsi/NOBATCH
#   TSI with batch sub-system: composed from tsi/SHARED and
#                              specific files from tsi/<specific_system>
#
nobatch_tsi=tsi/NOBATCH
shared_tsi=tsi/SHARED

#
# get the list of all TSIs
#
alltsis=`ls -d tsi/* tsi_contrib/* | sort`

#
# list all TSIs
#
echo ""
echo "Available TSI implementations are:"
echo ""
i=0
for tsi in $alltsis; do
  if [ $tsi != tsi/SHARED ]; then
    i=`expr $i + 1`
    echo "$i: $tsi"
  fi
done

#
# let the user select one TSI for installation
#
echo ""
echo "The installation will copy required files into a new directory."
echo "Files common to all TSI implementations are copied from tsi/SHARED."
echo "The TSI without a batch sub-system requires also files from tsi/NOBATCH."
echo "Each TSI with a batch sub-system needs the files from tsi/<specific_system>."
echo ""
echo "Select a TSI to install (enter number)"
read input
i=0
for tsi in $alltsis; do
  if [ "$tsi" != "$shared_tsi" ]; then
    i=`expr $i + 1`
    if [ $i = $input ]; then
      break
    fi
  fi
done

#
# get the installation directory from the user
#
installdir=`echo $tsi | sed 'y+/+_+'`
echo ""
echo "Enter installation directory for $tsi (relative or absolute path) [$installdir]"
read indir
if [ "$indir" != "" ]; then
  installdir=$indir
fi

#
# let the user confirm the installation
#
confirm_tsi() {
  echo ""
  echo "Please confirm installation of $1 into directory $2 by typing y/n [y]"
  read yesno
  case $yesno in
    y*|Y*) echo "";;
    "") echo "";;
    n*|N*) echo "Installation stopped because of missing confirmation"; exit 1;;
    *) echo "Installation stopped because of doubtful confirmation"; exit 1;;
  esac
}
confirm_tsi $tsi $installdir

#
# test existence of $installdir
#
if [ -d $installdir ]; then
  echo "Installation directory $installdir already exists"
  confirm_tsi $tsi $installdir
  echo ""
  echo "Files from $installdir will be overwritten"
  echo ""
  chmod -R u+w $installdir
elif [ -f $installdir ]; then
  echo "Installation directory $installdir identical to existing file."
  echo "Installation stopped because of doubtful confirmation,"
  echo "remove file $installdir first."
fi

#
# copy files from directory $1 to $2
#
copy_dir() {
  for file in $1/*; do
    echo "Copy $file to $2/`basename $file`"
    cp $file $2
  done
}

#
# generate installation directory
#
mkdir -p $installdir

#
# copy shared files
#
echo "Copy shared files (common to all installations) first:"
copy_dir $shared_tsi $installdir
#
# copy normal versions of specific files in case of certain TSI contributions
#
case $tsi in
  tsi_contrib/solaris_ZIB|tsi_contrib/superux_CSCS|tsi_contrib/unicos_ZIB|tsi_contrib/vpp_LRZ)
    echo ""
    echo "Copy (overwrite) specific files:"
    ;;
esac
case $tsi in
  tsi_contrib/solaris_ZIB)
    copy_dir tsi/NOBATCH $installdir
    ;;
  tsi_contrib/superux_CSCS)
    copy_dir tsi/superux $installdir
    ;;
  tsi_contrib/unicos_ZIB)
    copy_dir tsi/unicos $installdir
    ;;
  tsi_contrib/vpp_LRZ)
    copy_dir tsi/vpp $installdir
    ;;
esac
#
# copy specific files
#
echo ""
echo "Copy (overwrite) specific files:"
copy_dir $tsi $installdir

#
# restrict file permissions to owner
#
chmod 711 $installdir
chmod 600 $installdir/*
chmod 644 $installdir/tsi_ls

echo ""
echo "##########################################################"
if [ "$tsi" = "$nobatch_tsi" ]; then
  echo "Copy one of the following files to $installdir/tsi"
  for file in $installdir/tsi.*; do
    echo "	$file"
  done
fi
echo "Finish installation by editing $installdir/tsi."
echo "Execute \"Install_permissions.sh $installdir\" after update."
echo "Finally start the TSI as described in the README file"
echo "##########################################################"
echo ""
echo "NB: When porting to new platform all specific files have to be adapted!"
echo ""
