package Submit;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(submit);

use File::Path qw(mkpath);

use Reporting qw(debug_report failed_report ok_report start_report command_report report_and_die);

use strict;

# Submit the script to the BSS

# arg 1 = The script (this is called because the script contains the string "#TSI_SUBMIT");
#
# Returns void to TSI
#         on success returns the BSS identifier assigned to the job
#         on fail return a message 
#
# SPECIALISATION FOR WITHOUT BATCH SUB-SYSTEM

BEGIN {
    # Set a unique prefix for this invocation for all files
    # that it creates (based on process id)
    $Submit::counter=$$*100;   # used to give our processes a Unicore identifier

	# Always cd to some neutral place when not doing something
	$Submit::neutral_dir = $ENV{PWD};
	if($Submit::neutral_dir =~ m/(.*)/s) {
		$Submit::neutral_dir = $1;
	}

}

sub submit {

    # Reap children to stop zombies (PP p240)
    # Is this really necessary after Perl 5.004?
    local $SIG{CHLD} = sub { wait }; 

    my $script = shift;

	# Correction for $HOME or $USER in root directories SvdB 11 Feb 2002
	# Substitute for these values as the executed script's
	# environment will not (and if it did the values will
	# be wrong). This is safe as thes script seen here is
	# completely generated by the NJS and so we will not be
	# messing with any user created code.

	$script =~ s/\$USER/$ENV{USER}/g;
	$script =~ s/\$HOME/$ENV{HOME}/g;

	my $outcome_dir;
	my $uspace_dir;
    my $interactive =   "";
    my $job =   "";

	$_ = $script;
	while(/#TSI_(\S+) (.*)\n/g) {
		$outcome_dir = $2 if $1 eq "OUTCOME_DIR";
		$uspace_dir  = $2 if $1 eq "USPACE_DIR";
        $interactive =   $2 if $1 eq "PREFER_INTERACTIVE";
        $job = $2 if $1 eq "JOBNAME";
	}

	if($interactive eq "true") {
		start_report("Interactively executing a job");
	}
	else {
		start_report("Submitting a job to the pseudo-BSS");
	}

    # We are going to run this in a separate process
    # so do it as soon as possible (PP p167)

    my $jobid;
    my $pid;

    # First make a unique Job Identifier (as we
    # do not know process ID until after the fork and the
    # child needs the identifier
    $jobid=$Submit::counter;
    $Submit::counter++;

	# chdir here so that failures can be reported to NJS
	if(chdir($uspace_dir) == 0) {
		failed_report("Could not cd to Uspace $uspace_dir because $!");
		return 0;
	}

	# make sure that the outcome dir exists
	mkpath $outcome_dir, 0, 0700 unless -e $outcome_dir;
	chmod 0700, $outcome_dir;

	    my $cmds_file_name;

	if($interactive) {
		$cmds_file_name = "UNICORE_".$jobid."_Commands";
		open(EMSCRIPT,">$cmds_file_name");
		print EMSCRIPT $script;
		close(EMSCRIPT);
		`chmod 700 $cmds_file_name; ./$cmds_file_name > $outcome_dir/stdout 2> $outcome_dir/stderr; rm  $cmds_file_name`;
		chdir $Submit::neutral_dir;
		ok_report();
		return;
	}

	    $cmds_file_name = "UNICORE_".$jobid."_".$job;

    FORK: {
        if($pid = fork) {

	    if(chdir($Submit::neutral_dir) == 0) {
		debug_report("Could not cd to neutral dir $Submit::neutral_dir because $!");
	    }

            # Parent
            # Succeeded, return Job Identifier
            debug_report("Job submitted OK. Identifier: $jobid $cmds_file_name");
            print main::CMD_SOCK "$jobid\n";

        } elsif(defined $pid) { 

            # Child

            # Write the commands to a file
            open(EMSCRIPT,">$cmds_file_name");
            print EMSCRIPT $script;
            close(EMSCRIPT);

            # Do the work, cleaning up too (output file names match expectations from NQS)
            exec "chmod 700 $cmds_file_name; ./$cmds_file_name > $outcome_dir/stdout 2> $outcome_dir/stderr; rm  $cmds_file_name";
#            exec "chmod 700 $cmds_file_name; ./$cmds_file_name > UNICORE.o$jobid 2> UNICORE.e$jobid; rm  $cmds_file_name";
#            exec "chmod 700 $cmds_file_name; ./$cmds_file_name > UNICORE.o$jobid 2> UNICORE.e$jobid;";


        } elsif ($! =~ /No more process/) {
            # sensible to try again
            sleep 5;
            redo FORK;
        } else {
            # que?
            report_and_die("Can't fork for a submit: $!");
        }
    }
}
#
# $Id: Submit.pm,v 1.17 2002/04/09 17:42:19 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
# 24-07-01 SvdB Put stdout and stderr in Outcome dir, use Uspace for script file
# 21-08-01 Sven van den Berghe fecit Fixed neutral dir (cleaned for root too)
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
