package GetDirectory;

# This version is rqeuired to support NJS 4.0.3build1 and later (23 July 2003, SvdB)
# Allows returning of single files

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(get_directory);

use Reporting qw(start_report failed_report ok_report debug_report report_and_die);

use strict;

# Writes to the NJS COMMAND and DATA sockets

# Return a directory and its contents (recursively, following
# symbolic links) to the NJS
#
# Called in response to a TSI_GETDIRECTORY request from the NJS
#
# arg1: text sent by the NJS. this is processed for:
#           TSI_DIRECTORY - directory to return
#
# returns: void to TSI
#
#          takes over NJS-TSI protocol to send files.
#
sub get_directory {

    my $from_njs = shift;

    my $root_dir;

    # Extract the word following the marker
    ($root_dir) = ($from_njs =~ /.*#TSI_DIRECTORY (.+)\n/);

     # Tell NJS that we are ready and will be sending file data
    print main::CMD_SOCK "TSI_OK\n";
    print main::CMD_SOCK "ENDOFMESSAGE\n";

    $GetDirectory::failed = 0;   # This is deliberately global for called subs

    $root_dir =~ s/\$USER/$ENV{USER}/g;
    $root_dir =~ s/\$HOME/$ENV{HOME}/g;

    print main::CMD_SOCK "$root_dir\n";
    debug_report("Returning expanded directory root: $root_dir\n");

	if( -d "$root_dir") {
		return_directory("$root_dir");
	}
	else {
		return_a_file("$root_dir");
	}

    ok_report("Get directory") if $GetDirectory::failed == 0;
}


# Return a directory to the NJS (internal)

sub return_directory() {

    return if $GetDirectory::failed == 1;

    my $directory = shift;

    start_report("Sending contents of $directory to the NJS");

    if(! -e $directory) {
        $GetDirectory::failed = 1;
        failed_report("Directory does not exist: $directory");
        return;
    }
    if(! -r $directory) {
        $GetDirectory::failed = 1;
        failed_report("Cannot read directory: $directory");
        return;
    }
    if(! -x $directory) {
        $GetDirectory::failed = 1;
        failed_report("Cannot execute directory: $directory");
        return;
    }

    # Process this directory, doing any subs found, returning files
    my $file;
    local *DIR;

    opendir(DIR, $directory);   # this has gotta succeed after tests above
    while( defined($file = readdir(DIR))) {
        next if $file =~ /^\.\.?$/;   # skip . and ..
        if( -d "$directory/$file") {
            return_directory("$directory/$file");
        }
        else {
            return_a_file("$directory/$file");
        }

        last if $GetDirectory::failed == 1;
    }

    closedir(DIR);
}



# Return a file to the NJS (internal)

sub return_a_file {

    return if $GetDirectory::failed == 1;               # Allow a single failure during this call

    my $file_name = shift;

    if( ! open(FILE, "<$file_name") ) {
        failed_report("File cannot be opened: $file_name");
        $GetDirectory::failed = 1;
        return;
    }

	# Work out the file mode: it is open readable so must be at least 4
	my $mode = 4;
	$mode += 2 if -w $file_name;
	$mode += 1 if -x $file_name;

	if( -p $file_name) {

		print main::CMD_SOCK "$file_name\n-1 $mode\n";
		debug_report("Returning fifo: $file_name $mode\n");

		my $len;
		my $buf;
		my $offset;
		my $written;

		while($len = sysread FILE, $buf, $main::BUFFER_SIZE) {
			if(!defined $len) {
				die "Read error: $!:\n";
			}
			print main::CMD_SOCK "$len\n";
			$offset = 0;
			while($len) {  # handling partial writes
				$written = syswrite main::DATA_SOCK, $buf, $len, $offset;
				report_and_die("Portfolio copy error writing to NJS: $!") unless defined $written;
				$len -= $written;
				$offset += $written;
			};
		};
		print main::CMD_SOCK "0\n";     # 0 is EOF from sysread
	}
	else {
		my $size = -s $file_name;

		print main::CMD_SOCK "$file_name\n$size $mode\n";
		debug_report("Returning file: $file_name $mode $size bytes\n");

		my $len;
		my $buf;
		my $offset;
		my $written;

		while($len = sysread FILE, $buf, $main::BUFFER_SIZE) {
			if(!defined $len) {
				die "Read error: $!:\n";
			}
			$offset = 0;
			while($len) {  # handling partial writes
				$written = syswrite main::DATA_SOCK, $buf, $len, $offset;
				report_and_die("Portfolio copy error writing to NJS: $!") unless defined $written;
				$len -= $written;
				$offset += $written;
			};
		};
	}

    close(FILE);

}
#
# $Id: GetDirectory.pm,v 1.17 2002/04/09 17:42:08 svdb Exp $
#
# 31-03-03 Sven van den Berghe FLE Handle FIFOs (NJS release 4.0.2 and later)
# 04-05-01 Sven van den Berghe fecit Created from 3.2 TSI for Unicore 3.5
# 23-07-01 SvdB Transfer file mode information
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2003
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
