package JobControl;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(abort_job cancel_job hold_job resume_job);

use Reporting qw(start_report failed_report ok_report command_report);

use strict;

# Generic BSS job control functions

#
# Assumes that the following variables have been set up 
# before a call:
#
# $main::site_sez_no_holds: true if jobs on BSS cannot be held
#
# $main::abort_cmd:  command to execute to abort a BSS job (with process id asa argument)
# $main::cancel_cmd:  command to execute to cancel a BSS job (with process id asa argument)
# $main::hold_cmd:  command to execute to hold a BSS job (with process id asa argument)
# $main::resume_cmd:  command to execute to resume a BSS job (with process id asa argument)


# Abort a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub abort_job {

    my $from_njs = shift;

    my ($requestid, $output, $command);

    ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

    start_report("Aborting request $requestid");

    $command = "$main::abort_cmd $requestid\n";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {
        ok_report($output);
    }
}



# Cancel a job on the BSS (abort plus dispose of output files)
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub cancel_job {

    my $from_njs = shift;

    my ($requestid, $output, $command);

    ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

    start_report("Cancelling request $requestid");

    $command = "$main::cancel_cmd $requestid\n";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {
        ok_report($output);
    }
}


# Hold a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub hold_job {

    if($main::site_sez_no_holds) {
        ok_report("No holding of jobs on BSS")
    }
    else {

        my $from_njs = shift;

        my ($requestid, $output, $command);

        ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

        start_report("Holding request $requestid");

        $command = "$main::hold_cmd $requestid\n";
 
        command_report($command);
 
        $output = `($command) 2>&1`;
 
        if($? != 0) {
            failed_report($output);
        }
        else {
            ok_report($output);
        }
    }

}

# Resume a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub resume_job {

    if($main::site_sez_no_holds) {
        ok_report("No holding of jobs on BSS")
    }
    else {

        my $from_njs = shift;

        my ($requestid, $output, $command);

        ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

        start_report("Resuming request $requestid");

        $command = "$main::resume_cmd $requestid\n";
 
        command_report($command);
     
        $output = `($command) 2>&1`;
 
        if($? != 0) {
            failed_report($output);
        }
        else {
            ok_report($output);
        }
    }
}
# $Id: JobControl.pm,v 1.16 2002/04/09 17:42:09 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
