package ExecuteScript;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(execute_script);

use Reporting qw(start_report failed_report ok_report);

use strict;

# Executes a script from the NJS "interactively"
#
# Called in response to a #TSI_EXECUTESCRIPT string found in the 
# NJS message.

BEGIN {
     $ExecuteScript::counter = 1; # Order DCE files sequentially
}

# arg 1 = Script to execute
#
# Returns void to TSI
#
#  success/failure to NJS
#
sub execute_script {
 
     my $script = shift;

     # Substitute for these values as the executed script's
     # environment will not (and if it did the values will
     # be wrong). This is safe as thes script seen here is
     # completely generated by the NJS and so we will not be 
     # messing with any user created code.

     $script =~ s/\$USER/$ENV{USER}/g;
     $script =~ s/\$HOME/$ENV{HOME}/g;

     # Create a file name to which scripts are written for running it under
     # DCE control. This needs to be unique as there may be
     # more than one TSI running, so base on the process id.

     $ExecuteScript::tsi_unique_file_name = "TSI_temp_DCE_file_$$.$ExecuteScript::counter";
     $ExecuteScript::counter++;

     open(DCESCRIPT,">$ExecuteScript::tsi_unique_file_name");
     print DCESCRIPT $script;
     close(DCESCRIPT);

     chmod 0700, $ExecuteScript::tsi_unique_file_name;

     start_report("Executing a script.");
 
#    my $output = `($script) 2>&1`;

     my $output = `(dce_login $ENV{USER} -k /usr/local/unicore_4.0/keytabfiles/$ENV{USER}.ktf -e $ExecuteScript::tsi_unique_file_name) 2>&1`;
 
     if($? != 0) {
        failed_report($output);
     }
     else {
        ok_report($output);
        print main::CMD_SOCK $output;
     }

     unlink $ExecuteScript::tsi_unique_file_name;
 }
#
# $Id: ExecuteScript.pm,v 1.15 2001/08/28 16:55:50 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
# 04-06-01 Sven van den Berghe fecit Replace $USER and $HOME
#
#                       Copyright 1999 to 2001 Fujitsu European Centre for Information Technology Ltd.
#  This is unpublished proprietary source code which is the property of Fujitsu European Centre for Information Technology Ltd.

