package Submit;

###############################################################################
#
# Loadleveler on AIX
#
# UNTESTED
#
# Based on V3.0 version developed for Idris
#
###############################################################################

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(submit);

use File::Path qw(mkpath);

use Reporting qw(debug_report failed_report ok_report start_report command_report report_and_die);

use strict;

# UXP/V specialisation

# Submit the script to the BSS

# arg 1 = The script (this is called because the script contains the string "#TSI_SUBMIT");
#
# Returns void to TSI
#         on success returns the BSS identifier assigned to the job
#         on fail return a message 
#
# The "stdfiles" function (in EndProcessing.pm) expects the stdout/stderr files 
# written by the job to be in the CWD of theTSI process, the submit here 
# must ensure this.
#

BEGIN {
    # --------------------------------------------------------------------
    # Create a file name to which scripts are written for the submits to
    # the batch sub-system. This needs to be unique as there may be
    # more than one TSI running, so base on the process id.

    $Submit::tsi_unique_file_name = "TSI_temp_file_$$";

    # Always cd to some neutral place when not doing something
    $Submit::neutral_dir = $ENV{PWD};
    if($Submit::neutral_dir =~ m/(.*)/s) {
        $Submit::neutral_dir = $1;
    }

}

sub submit {

    my $from_njs = shift;

	# Correction for $HOME or $USER in root directories SvdB 11 Feb 2002
	# Substitute for these values as the executed script's
	# environment will not (and if it did the values will
	# be wrong). This is safe as thes script seen here is
	# completely generated by the NJS and so we will not be
	# messing with any user created code.

	$from_njs =~ s/\$USER/$ENV{USER}/g;
	$from_njs =~ s/\$HOME/$ENV{HOME}/g;

    # Get the information from the NJS (embedded as comments)
    #
    # Not picking up defnitions of Software Resources sent as:
    # #TSI_SWR<name>

    # First clear all command line sections from previous iterations
    my $jobname =       "";
    my $outcome_dir =   "";
    my $uspace_dir =    "";
    my $time =          "";
    my $memory =        "";
    my $nodes =         "";
    my $processors =    "";
    my $fast_fs =       "";
    my $large_fs =      "";
    my $home_fast_fs =  "";
    my $home_large_fs = "";
    my $queue =         "";
    my $email =         "";
    my $interactive =   "";

    $_ = $from_njs;
    while(/#TSI_(\S+) (.*)\n/g) {
        $jobname =       $2 if $1 eq "JOBNAME";
        $outcome_dir =   $2 if $1 eq "OUTCOME_DIR";
        $uspace_dir =    $2 if $1 eq "USPACE_DIR";
        $time =          $2 if $1 eq "TIME";
        $memory =        $2 if $1 eq "MEMORY";
        $nodes =         $2 if $1 eq "NODES";
        $processors =    $2 if $1 eq "PROCESSORS";
        $fast_fs =       $2 if $1 eq "FASTFS";
        $large_fs =      $2 if $1 eq "LARGEFS";
        $home_fast_fs =  $2 if $1 eq "HOMEFASTFS";
        $home_large_fs = $2 if $1 eq "HOMELARGEFS";
        $queue =         $2 if $1 eq "QUEUE";
        $email =         $2 if $1 eq "EMAIL";
        $interactive =   $2 if $1 eq "PREFER_INTERACTIVE";
    }

	if($interactive eq "true") {
		start_report("Interactively executing a job");
	}
	else {
		start_report("Submitting a job to the BSS");
	}

    # Jobname
    # IGNORED by Karlsruhe script
    if($jobname eq "NONE" ) {
        $jobname = "# @ job_name = $main::default_job_name\n";
    }
    else {
        $jobname = "# @ job_name = $jobname\n";
    }

    # Queue 
    #
    # The IDB will send the queue name of NONE if it is
    # set up with a single queue whose name is "noname"

    if($queue eq "NONE" ) {
        $queue = "";
    }
    else {
        # $queue = "# @ class = $queue\n";
        # For Karlsruhe script
        $queue = "-q $queue";
    }

    # Job memory requirements in megabytes, this can be
    # either for the whole job, for each processor or
    # for each node depending on how the IDB is set up
    # (see PER_xxxx_LIMITS)
    #$memory = "# @ requirements =  (Memory >= $memory)\n";

    # For Karlsruhe script
    $memory = "-lm $memory";

    # Job time requirement. Wallclock time in seconds.
    #$time = "# @ cpu_limit = $time\n"; 

    # For Karlsruhe script
    $time = "-lt $time"; 

    # NOTE: With V3.5 the NJS support SMPs by always sending
    #       a node request and a processors per node request.
    #
    #       $nodes is number of nodes required
    #
    #       $processors (processors per node) is 1 if the system
    #       is not an SMP
    #       
    # Nodes count, NONE implies serial, otherwise a valid number
    #
    if($nodes eq "NONE") {
        #$nodes = "# @ job_type = serial\n";

        # For Karlsruhe script
        $nodes = "-lP 0";
    }
    else {
        #$nodes = "# @ job_type = parallel\n# @ node = $nodes,$nodes\n";

        # For Karlsruhe script
        $nodes = "-lP $nodes";
    }
        
    # Email (email address of recepient)
    # IGNORED by Karlsruhe script
    if($email eq "NONE") {
        $email = "";
    }
    else {
        $email = "# @ notification = always\n# @ notify_user = $email\n" ;
    }

    # Tell the BSS to put the batch job's stdout and stderr into these files
    print "NOTE NEEDS CHANGING TO TELL LOADLEVELER TO PUT STDOUT AND STDERR INTO SPECIFIED FILES\n";
    # my $stdout_loc = "# @ output = $outcome_dir/stdout \n";
    # my $stderr_loc = "# @ error = $outcome_dir/stderr \n";

    my $stdout_loc = "$outcome_dir" . "stdout";
    my $stderr_loc = "$outcome_dir" . "stderr";

    # cd to the Uspace directory, will write a file soon and
    # this means that there does not need to be a TSI working
    # directory
    if(chdir($uspace_dir) == 0) {
        failed_report("Could not cd to Uspace $uspace_dir because $!");
        return 0;
    }

    # Make sure that the Outcome directory is there for the stdout and stderr files
    mkpath $outcome_dir, 0, 0777 unless -e $outcome_dir;
    chmod 0777, $outcome_dir;

    # Put these requests into the script that is
    # submitted to Loadleveler
    # "# @ queue" is required before actual script by Loadleveler synrax
    #$from_njs = "#!/bin/sh\n$queue$processors$memory$email$time$jobname$stdout_loc$stderr_loc # @ que ue\n$from_njs";

    # my $command = "$main::submit_cmd $queue $nodes $time $memory $Submit::tsi_unique_file_name";
    my $command = "$main::submit_cmd $queue $nodes $time $memory $stdout_loc $stderr_loc $Submit::tsi_unique_file_name";

    command_report($command);

    open(EMSCRIPT,">$Submit::tsi_unique_file_name");
    print EMSCRIPT $from_njs;
    close(EMSCRIPT);

	if ($interactive eq "true"){
		my $command = "$Submit::tsi_unique_file_name";
		chmod 0700, $command;
		command_report($command);
		`./$command > $outcome_dir/stdout 2> $outcome_dir/stderr`;
		ok_report();
	}
	else {

    chmod 0777, $Submit::tsi_unique_file_name;

    # and execute the command
    my $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {

        # Succeeded, return Job Identifier

        # Previous, wrong anyway? e.g. sp5cws.rz.uni-karlsruhe.de.xxx
        # if($output =~ /The job "(.+?)\..*/) {
        #    # note the minimal match - for first "."

        # Match the complete jobid not just numeric - assumes that failed
        # submits are caught by return code of by the else block
        if($output =~ /The job "(\S+)" has been submitted.*/) {

            my $jobid = $1;

            debug_report("Job submitted OK. Identifier: $jobid");
            print main::CMD_SOCK "$jobid\n";

        }
        else {
            failed_report("Request id not found in: $output");
        }

     }
    }

#   unlink $Submit::tsi_unique_file_name;

    chdir $Submit::neutral_dir;

}

#
# $Id: Submit.pm,v 1.4 2001/08/28 16:55:45 svdb Exp $
#
# 16-08-01 Sven van den Berghe fecit Created from 3.5 AIXLL/Submit.pm
# 21-08-01 Sven van den Berghe fecit Fixed neutral dir (cleaned for root too)
#
#                       Copyright 1999 to 2001 Fujitsu European Centre for Information Technology Ltd.
#  This is unpublished proprietary source code which is the property of Fujitsu European Centre for Information Technology Ltd.
#
