package Submit;

###############################################################################
# UXP/V
#
#    developed on a VPP at fecit (tested) Sven van den Berghe
#    adapted for Hitachi sr8000 LRZ Muenchen Irene Geiseler (ige)
#
###############################################################################
# LRZ sr8000 needs:
# - to submit with the users login shell 
# - a submit -script with qsub-options
# - to submit to a partition, no to a queue
# - a prologue file in qsub-script (sub getloginfile)
# - modifications for CPMD and Gaussian98
# all LRZ-changes are marked with "ige"
##############################################################################

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(submit);

use File::Path qw(mkpath);

use Reporting qw(debug_report failed_report ok_report start_report command_report report_and_die);
use strict;

# UXP/V specialisation

# Submit the script to the BSS

# arg 1 = The script (this is called because the script contains the string "#TSI_SUBMIT");
#
# Returns void to TSI
#         on success returns the BSS identifier assigned to the job
#         on fail return a message 
#
# The "stdfiles" function (in EndProcessing.pm) expects the stdout/stderr files 
# written by the job to be in the CWD of theTSI process, the submit here 
# must ensure this.
#

BEGIN {
    # --------------------------------------------------------------------
    # Create a file name to which scripts are written for the submits to
    # the batch sub-system. This needs to be unique as there may be
    # more than one TSI running, so base on the process id.

    $Submit::tsi_unique_file_name = "TSI_temp_file_$$";
    # Always cd to some neutral place when not doing something
    $Submit::neutral_dir = $ENV{PWD};
    $Submit::user = $ENV{USER};
    if($Submit::neutral_dir =~ m/(.*)/s) {
        $Submit::neutral_dir = $1;
 
    }
print "neutral_dir : $Submit::neutral_dir\n";
}
### add sub getloginfile because we need a system prologue in the qsub-script ### ige
### becomes username and returns ". /etc/profile" or "source /etc/csh.login" ### ige
#
sub get_login_shell
{
 my ($new_name) = @_;
 my ($l_name, $l_passwd, $l_uid, $l_gid, $l_quota, $l_comment, $l_gcos, $l_dir, $l_shell);
 ($l_name, $l_passwd, $l_uid, $l_gid, $l_quota, $l_comment, $l_gcos, $l_dir, $l_shell) = getpwnam($new_name);
 return ($l_shell);
} # --------------------- end of sub get_login_shell,  ige
#
sub getloginfile
{
my ($user) = @_;
my $nqs_shell = "";
my $shstart = "/etc/profile";
my $cshstart = "/etc/csh.login";
 
             $nqs_shell = ". $shstart";
             $nqs_shell = "source $cshstart" if ($Submit::user_login_shell =~ /csh/);

return ($nqs_shell);
}  # ____________________ end of sub  getloginfile   ### ige

sub submit {
   my $from_njs = shift;
my ($from_njs_csh, $from_njs_xsh) = ("", "");        ### for csh|tcsh or sh|ksh|bash,  ige
###
    #
    
    # Get the information from the NJS (embedded as comments)
    #
    # Not picking up defnitions of Software Resources sent as:
    # #TSI_SWR<name>

    # First clear all command line sections from previous iterations
    my $jobname =       "";
    my $outcome_dir =   "";
    my $uspace_dir =    "";
    my $time =           0;
    my $memory =         0;
    my $nodes =          0;
    my $processors =     1;
    my $fast_fs =       "";
    my $large_fs =      "";
    my $home_fast_fs =  "";
    my $home_large_fs = "";
    my $queue =         "";
    my $email =         "";
    my $portofolio =    "";
    my $identity =      "";  ### ige
    my $user =          "";  ### ige
    my $proc =          "";  ### ige
    my $execute =       "";  ### ige
    my $softres =       "";  ### ige
#
$_ = $from_njs;
print "FROMNJS von HIER\n$from_njs\nBIS HIER\n";
#
  while (/#TSI_(\S+) (.*)\n/g) { 
        $jobname =       $2 if $1 eq "JOBNAME";
        $outcome_dir =   $2 if $1 eq "OUTCOME_DIR";
        $uspace_dir =    $2 if $1 eq "USPACE_DIR";
        $time =          $2 if $1 eq "TIME";
        $memory =        $2 if $1 eq "MEMORY";
        $nodes =         $2 if $1 eq "NODES";
        $processors =    $2 if $1 eq "PROCESSORS";
        $fast_fs =       $2 if $1 eq "FASTFS";
        $large_fs =      $2 if $1 eq "LARGEFS";
        $home_fast_fs =  $2 if $1 eq "HOMEFASTFS";
        $home_large_fs = $2 if $1 eq "HOMELARGEFS";
        $queue =         $2 if $1 eq "QUEUE";
        $email =         $2 if $1 eq "EMAIL";
        $identity =      $2 if $1 eq "IDENTITY";
        $softres =	 $2 if $1 eq "SWR";    
       
    }
 while (/(PF\-\S+\-FP)/g)  {                                       ### added,   ige
        $portofolio = $1;
     }
($user, $proc) = split /\s+/, $identity;
$Submit::user_login_shell = &Submit::get_login_shell($user);
#
my $msg_user = <<PRINT_MSG;
 \"\\n___________________________________________________\\nDear Hitachi-sr8000 user\\n\\nPlease use your login shell for shell scripts.\\nThis is required by our NQS-configuration.\\nPlease contact 'hlrb-admin\@lrz.de' if you like to change\\nyour login shell.\\nYour login shell is: $Submit::user_login_shell\\n___________________________________________________\\n\"
PRINT_MSG
#
my $gaus_prolog = "";
my $gauss_sh = ". /usr/local/sys/gaussian/unicore_start.sh";
my $gauss_csh = "source /usr/local/sys/gaussian/unicore_start.csh";
my $path = "";
$path = 'PATH=$PATH:'; 
$path = 'set path = ($path ./)'  if ($Submit::user_login_shell =~ /\/csh|tcsh/);
print "SOFTRES: $softres\n";
# normal with softres, but now we haven't any in $from_njs 
# if ($softres  =~ /Gaussian98/)
# {
 $gaus_prolog = $gauss_sh;
 $gaus_prolog = $gauss_csh if ($Submit::user_login_shell =~ /\/csh|tcsh/);
# }
print "Gaus Prolog: $gaus_prolog\n"; 
print "MY PATH IS $path YUHU!!\n";
my $print_msg = '/usr/bin/printf' . "$msg_user"; 
s/#!.*\n/#!$Submit::user_login_shell\n$print_msg\n$path\n/;             ### we replace the sh with the login-shell, ige
### we replace "\n" with " " and don't need a new line like this ' " " "` '...    ige
#
s/\/usr\/bin\/tr -s \"\n\" \" \"/\/usr\/bin\/tr -s \"\\n\" \" \"/g;
s/\"Execution failed: Task incarnation set non-zero error code\n\"/\"Execution failed: Task incarnation set non-zero error code\\n\"/g;
# 
 
### we have to work in the same shell (". " or "source") - if we don't have a perl script,  ige 
#
### only the shell is changed if login shell is sh|ksh|bash,  ige
$from_njs_xsh = $_;
#
### this is a "$from_njs"-csh script for a csh/tcsh login shell, ige
### only necessary, if login shell is csh/tcsh
#
my $print_end = <<PRINT_END_TEXT;
# /usr/bin/printf "\\nUNICORE - end of user output on stderr\\n" \>\>\&\! ${outcome_dir}stderr
/usr/bin/printf "\\nUNICORE - end of user output on stderr\\n" \> \&2
/usr/bin/printf "\\nUNICORE - end of user output on stdout\\n"
echo "UNICORE EXIT STATUS" \$the_last + \>\> ${outcome_dir}stderr 
PRINT_END_TEXT
my $sig_error = 'set the_last = 57' . "\n" . 'exit 1' . "\n";
my $sig_handler = 'exit 0'. "\n" . 'signal:' . "\n" . $sig_error;
#
 if ($Submit::user_login_shell =~ /csh|tcsh/)  { 
$_ = $from_njs_xsh;
# we don't need this..
s/trap.*\n|\s*then\s*\n|on_exit ().*\n|\s*the_end=.*\n//g;
#
# we don't need the function on_exit to, but "onintr"
s/\s*\{\s*(\s*\n.*)*\s*\}\s*\n/\nonintr signal\n/g;

# the environement for csh
s/UNICORE_JOB=(.*); .*\n/setenv UNICORE_JOB $1\n/;
s/UC_TARGET=(.*); .*\n/setenv UC_TARGET $1\n/;
s/GAUSS_EXEDIR=(.*); .*\n/setenv GAUSS_EXEDIR $1\n/;
s/UC_NODES=(.*); .*\n/setenv UC_NODES $1\n/;
s/UC_PROCESSORS=(.*); .*\n/setenv UC_PROCESSORS $1\n/;
s/USPACE_HOME=(.*); .*\n/setenv USPACE_HOME $1\n/;
s/JOBTYPE=(.*); .*\n/setenv JOBTYPE $1\n/;
# csh syntax for the rest...
s/(\w+)=(.*)\n/set $1 = $2\n/g;
s/the_last=\$\?/the_last = \$status/;
s/if \[ (.*) \]/if \($1\) then\n/g;
s/fi\s*\n/endif\n/g;
s/ 1\>\&2\n/\>\&\! ${outcome_dir}stderr\n/g;
$from_njs_csh = $_;

# and the mark for the signal handler
$from_njs_csh = $from_njs_csh . $print_end . $sig_handler;
} 
#
### end of csh|tcsh "$from_njs"          ### ige 

    start_report("Submitting a job to the BSS");
my $tag = '#@$-';                              ### for NQS startscript, ige
my $endtag = '#@$';                            ### mark the end of NQS directives, ige
my $other_opt = "";                            ### other needed options for NQS startscript, ige
my $prologue = "";                             ### if prologue eq "", $USER doesn't exist  in /etc/passwd, ige  
$prologue = Submit::getloginfile($user) if ($user);   ### give the standard prologue for the login-shell, ige
### without prologue not submit will be possible, ige ###   
  if ($prologue eq "")  {
  debug_report("Can't get prologue for the login-shell, for this user User $Submit::user");
  return 1;
  }                                     
###  
my ($sub_queue, $sub_jobname, $sub_time, $sub_nodes, $sub_memory) = ("", "", "", "", ""); 
     # Jobname (source man page)
    if($jobname eq "NONE" ) {
        $sub_jobname = $tag . "r " .  $main::default_job_name . "\n";    ### ige
    }
    else {
        $sub_jobname = $tag . "r " . $jobname . "\n";                    ### ige
    }

    # Queue (source man page)
    #
    # The IDB will send the queue name of NONE if it is
    # set up with a single queue whose name is "noname"

        $sub_queue = "";                            ### we submit only with "-P partition name", ige

    # Job memory requirements in megabytes, this can be
    # either for the whole job, for each processor or
    # for each node depending on how the IDB is set up
    # (see PER_xxxx_LIMITS)

    # Note the -ls below, need a minimum size
    $memory = 200 if ($memory < 200);                            ### less is dangerous,  ige
    $time = "800" if($time < 800);                               ### less is dangerous,  ige

### submit in PARALLEL or MEMORY-partition, ige ###   
    $sub_queue = $tag . "P PARALLEL\n" if ($memory <= 6400 && $nodes >= 1 && $processors > 1);              
    $sub_queue = $tag . "P MEMORY\n" if ($memory > 6400 && $nodes >= 1  && $nodes <= 4 && $processors > 1);
    failed_report("You can't get $memory MB for more as 4 Nodes") if ($memory > 6400 && $nodes > 4); 
    $sub_memory = $tag . "lM " . $memory . "MB\n";                        ### ige
    # Job time requirement. Wallclock time in seconds.
    $sub_time = $tag . "lE " . $time . "\n";                              ### ige 
     
    # $nodes is number of nodes required
    #
    # $processors (processors per node) is 1 if the system
    # is not an SMP
    #       
    # Nodes count, NONE implies serial, otherwise a valid number
    #
#    if($nodes eq "NONE" && $processors >= 1 && $processors <= 8) {
#
### submit in SCALAR, ige ####                                          
     if($nodes <= 1 && $processors == 1) {                                           
         $sub_nodes = "";                                                  ### no nodes entry for SCALAR, ige 
         $memory = 800 if ($memory > 800);                                      
         $sub_memory = $tag . "lM " . $memory . "MB\n";                    ### ige 
         $sub_queue = $tag . "P SCALAR\n";                                                
         $time = 7200 if ($time > 7200);                                   ### max 2 hours for scalar jobs, ige
         $sub_time = $tag . "lE $time\n";     
         $other_opt = $tag . "nr\n" . $tag . "J SS\n";                     ### only SS for SCALAR, ige   
     }
     else {
         $sub_nodes = $tag . "N " . $nodes . "\n";                         ### ige 
     }
### for Denver-Tests, ige ###
     if ($softres eq "CPMD" && $nodes <= 4 && $memory <= 6400)
     {
 	$sub_queue = $tag . "P IAPAR\n";
     }
     elsif ($softres eq "CPMD" && ($nodes > 4 || $memory > 6400) )
     {
 	failed_report("The limits for CPMD are nodes <= 4 and memory <= 6400. Please try again");
     }
# we haven't $softres for Gaussian in $from_njs 
# if ($softres =~ /Gaussian98/)
# {
# $sub_queue = $tag . "P IAPAR\n";
# $sub_queue = $tag . "P MEMORY\n" if ($memory > 6400 && $nodes >= 1  && $nodes <= 4 && $processors > 1);
# $sub_nodes = $tag . "N 1\n";
# } 
#
    if ($nodes <= 4 && $memory <= 6400 && $processors > 1 ) 
    {
       $sub_queue = $tag . "P IAPAR\n";
       $other_opt = $tag . "nr\n" . $tag . "J E8S\n";
    }
##
# Email source
    if($email eq "NONE") {
        $email = "";
    }
    else {
    #    $email = $tag . "mu " . $email . "\n" . $tag . "me\n";                             ### ige
         $email = $tag . "me\n";                         ### because max lenght 15!!!
    }
    $other_opt = $tag . "nr\n" . $tag . "J E8E\n" if ($sub_queue =~ /PARALLEL|MEMORY|IAPAR/);    ### ige

    # Tell the BSS to put the batch job's stdout and stderr
    # into these files
    my $stdout_loc = $tag . "o " . $outcome_dir . "stdout" . "\n";
    my $stderr_loc = $tag . "e " . $outcome_dir . "stderr" . "\n";

    # cd to the Uspace directory, will write a file soon and
    # this means that there does not need to be a TSI working
    # directory
    if(chdir($uspace_dir) == 0) {
        failed_report("Could not cd to Uspace $uspace_dir because $!");
        return 0;
    }

    # Make sure that the Outcome directory is there for the stdout and stderr files
    mkpath $outcome_dir, 0, 0700 unless -e $outcome_dir;
    chmod 0700, $outcome_dir;

### qsub options + prologue in $script,  ige ###
# now all users will become Gaussian environement... 
my $script = $sub_queue . $sub_nodes . $sub_time . $sub_jobname . $sub_memory . $stdout_loc . $stderr_loc . $email . $other_opt .$endtag . "\n" . $prologue  . "\n" . $gaus_prolog . "\n"; 
#
    my $command = "$main::submit_cmd $Submit::tsi_unique_file_name";       
    command_report($command);
    open(EMSCRIPT,">$Submit::tsi_unique_file_name");
    print EMSCRIPT "$script";

### then submit wich the login-shell,  ige ###
     if ($Submit::user_login_shell =~ /\/sh|ksh|bash/)
     {
       # print EMSCRIPT 'PATH=$PATH:\n';   # Add . to PATH for this script (from Sven)
        print EMSCRIPT "$from_njs_xsh";
 print "Ich submitiere :\n$from_njs_xsh"; ### nur zum Testen, ige  
     }
     elsif ($Submit::user_login_shell =~ /csh|tcsh/)
     {
      #   print EMSCRIPT 'set path = ( $path ./ )\n';   # Add . to PATH for this script (from Sven)
         print EMSCRIPT "$from_njs_csh";
 print "Ich submitiere :\n$from_njs_csh"; ### nur zum Testen, ige 
     }
close(EMSCRIPT);  
             

    # and execute the command
# my $liste = `ls -l`;
# my $curdir = `pwd`;
# print "$curdir\n$liste\n";
  my $output = `($command) 2>&1`;
  print " Ergebniss submit: $output\n";
# print "$curdir\n$liste\nEND SUBMIT\n";   
    
   # Parse output
    if($? != 0) {                                
    failed_report($output);                     
    }                                           
    else {                                       

        # Succeeded, return Job Identifier
        # looking for line Request xxxx.blah submitted successfully (want the xxxx)

        if($output =~ /Request (.+?)\..*/) {
            # note the minimal match - for first "."
            my $jobid = $1;
            # qsub does not report a failure code so we need to
            # check here that we got an integer. If we did assume
            # OK, otherwise report a failure
            my $res = $jobid;
            $res =~ s/[0-9]//g;
            if (length($res) == 0 ) {
                debug_report("Job submitted OK. Identifier: $jobid");
                print main::CMD_SOCK "$jobid\n";
            }
            else {
                failed_report("Job submit failed?: $jobid $output");
            }
        }
        else {
            failed_report("Request id not found in: $output");
        }

   }                                               #

 #   unlink $Submit::tsi_unique_file_name;                     ###  ige 

    chdir $Submit::neutral_dir;

}

#
# $Id: Submit.pm,v 1.11 2001/08/28 16:56:02 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
# 21-08-01 Sven van den Berghe fecit Fixed neutral dir (cleaned for root too)
#
#                       Copyright 1999 to 2001 Fujitsu European Centre for Information Technology Ltd.
#  This is unpublished proprietary source code which is the property of Fujitsu European Centre for Information Technology Ltd.
#
